/*
 * @DEC_COPYRIGHT@
 */
/*
 * HISTORY
 * $Log: mmbasic.h,v $
 * Revision 1.2.11.2  1997/10/21  18:12:18  Chip_Dancy
 * 	Add ifdefs to compile under c++.
 * 	[1997/10/21  18:08:49  Chip_Dancy]
 *
 * Revision 1.2.9.2  1996/04/02  16:36:31  Jim_Ludwig
 * 	Fix problem with BYTE definition. Xmd.h also defines this
 * 	type and strict checking by the compiler doesn't allow
 * 	multiple definitions, even if they are the same. BOOL is
 * 	also added here.
 * 	[1996/04/02  16:36:19  Jim_Ludwig]
 * 
 * Revision 1.2.7.2  1996/02/26  17:31:39  Chip_Dancy
 * 	Remove typedef for SIZE; replaced with MME_SIZE in mmddk.h;
 * 	Fixes part of PTT-163; conflict with C++.
 * 	[1996/02/26  17:30:21  Chip_Dancy]
 * 
 * Revision 1.2.5.2  1995/11/29  10:41:46  Jim_Ludwig
 * 	Strip the head info that says this doesn't compile.
 * 	[1995/11/29  10:41:29  Jim_Ludwig]
 * 
 * Revision 1.2.2.11  1993/08/23  15:54:34  Chip_Dancy
 * 	Replace slash-slash style comments with slash-asterisk style.
 * 	[1993/08/23  15:54:17  Chip_Dancy]
 * 
 * Revision 1.2.2.10  1993/07/15  13:27:03  Chip_Dancy
 * 	Remove use of /* to avoid lint errors.*/
 * 	[1993/07/15  13:25:41  Chip_Dancy]
 * 
 * Revision 1.2.2.9  1993/05/20  13:54:07  Jim_Ludwig
 * 	Change BOOL to match X's
 * 	[1993/05/20  13:53:35  Jim_Ludwig]
 * 
 * Revision 1.2.2.8  1993/04/26  16:52:44  Allen_Arumainayagam
 * 	Added Uint64 back!
 * 	[1993/04/26  16:52:26  Allen_Arumainayagam]
 * 
 * Revision 1.2.2.7  1993/04/26  16:17:47  Allen_Arumainayagam
 * 	      Defined more types, this file was depending on other
 * 	      header files to provide data types.
 * 	      Redefined the basic data types and windows data types.
 * 	[1993/04/26  15:55:56  Allen_Arumainayagam]
 * 
 * Revision 1.2.2.6  1993/04/21  22:47:38  Jim_Ludwig
 * 	Add DECLARE_HANDLE
 * 	Add PASCAL
 * 	[1993/04/21  22:43:33  Jim_Ludwig]
 * 
 * Revision 1.2.2.5  1993/04/20  16:51:53  Jim_Ludwig
 * 	Null out FAR and NEAR
 * 	[1993/04/20  16:46:43  Jim_Ludwig]
 * 
 * Revision 1.2.2.4  1993/04/15  16:57:45  Jim_Ludwig
 * 	BOOL needed fixing
 * 	[1993/04/15  16:57:30  Jim_Ludwig]
 * 
 * Revision 1.2.2.3  1993/04/15  16:37:17  Jim_Ludwig
 * 	type corrections
 * 	[1993/04/15  16:36:57  Jim_Ludwig]
 * 
 * Revision 1.2.2.2  1993/04/14  15:13:22  Jim_Ludwig
 * 	Change DWORD to Int32 where it belongs.
 * 	Interfaces that rely on a pointer being held with DWORD must
 * 	either change their semantics or pick another type.
 * 	[1993/04/14  15:13:01  Jim_Ludwig]
 * 
 * Revision 1.2  1993/03/29  17:16:31  root
 * 	Initial load of project
 * 
 * $EndLog$
 */
/*
**++
**
** FACILITY: Workstation Multimedia (WMM) v1.0
**
** FILE NAME: mmbasic.h
**
** MODULE NAME:  mmbasic
**
** MODULE DESCRIPTION:
**       Basic definitions used by the ALPHA_MM system.
**
**--
*/
#ifndef H_MMBASIC
#define H_MMBASIC 1

#ifdef __cplusplus
extern "C" {	/* Assume C declarations for C++ */
#endif	/* __cplusplus */
    
/*
** [Include Files]
*/
/* None */
/*
** Constant Definitions
*/
#define CONST               const
#define CALLBACK
#define APIENTRY
#define WINAPI   	APIENTRY
#define FAR
#define NEAR
#define PASCAL

/*
** Data Structure Definitions
*/
/* basic data type for ALPHA MM system */
typedef char                Int8;
typedef short               Int16;
typedef int                 Int32;
typedef long                Int64;

typedef unsigned char       Uint8;
typedef unsigned short      Uint16;
typedef unsigned int        Uint32;     
typedef unsigned long       Uint64; 


typedef char                Char8;
typedef short               Char16;

/* Windows specific data types */
typedef Uint32      	    DWORD;
#ifndef XMD_H /* Xmd.h also defines this... We could have deferred
	       * to that file but that file isn't included normally.
	       * Mrm does include it. We still have to define it
	       * here since someone might not include Xmd.h. -std1
	       * will result in Errors if this is typedef'd more than
	       * once.
	       * So, if you need to include X11/Xmd.h, include it before
	       * any mme header files.
	       */
typedef unsigned char 		    BOOL;
typedef Uint8       	    BYTE;
#endif
#ifndef TRUE
#define TRUE	1
#endif
#ifndef FALSE
#define FALSE	0
#endif
#ifndef NULL
		#undef NULL
#define NULL  0
#endif
typedef Uint16      	    WORD;
typedef Int32               INT;
typedef Int64               LONG;
typedef float               FLOAT;
typedef Uint32        	    UINT;
typedef Char8 		    CHAR;
typedef Int16 		    SHORT;

typedef Uint32        	    *PUINT;
typedef FLOAT               *PFLOAT;
typedef BOOL 	            *PBOOL;
typedef BOOL                *LPBOOL;
typedef BYTE                *PBYTE;
typedef BYTE                *LPBYTE;
typedef INT                 *PINT;
typedef INT                 *LPINT;
typedef WORD                *PWORD;
typedef WORD                *LPWORD;
typedef LONG                *LPLONG;
typedef DWORD               *PDWORD;
typedef DWORD               *LPDWORD;
typedef void		    VOID;
typedef void                *LPVOID;

/* Types use for passing & returning polymorphic values */
typedef UINT                WPARAM;
typedef LONG                LPARAM;
typedef UINT                MMRESULT;

/* Handle is 32 bit UINT */
typedef UINT HANDLE;                

typedef Char16 WCHAR;    /* wc,   16-bit UNICODE character */

typedef WCHAR *PWCHAR;
typedef WCHAR *LPWCH, *PWCH;
typedef CONST WCHAR *LPCWCH, *PCWCH;
typedef WCHAR *NWPSTR;
typedef WCHAR *LPWSTR, *PWSTR;

typedef CONST WCHAR *LPCWSTR, *PCWSTR;
typedef CHAR *LPSTR, *PSTR;
typedef CHAR *LPCSTR;

#define DECLARE_HANDLE(handle) typedef HANDLE handle
/*
** Exported Global Variables
*/
/* None */

/*
** Pseudo Functions (#defined macros)
*/
/* None */

/*
** Exported Function Proto-types
*/
/* None */

#ifdef __cplusplus
}                       /* End of extern "C" { */
#endif	/* __cplusplus */

#endif   /* H_MMBASIC */

