
/*   Copyright 2001 Compaq Computer Corporation*/

/*  Compaq and the Compaq logo Registered in U.S. Patent and Trademark Office.*/

/*  Confidential computer software. Valid license from Compaq required for*/
/*  possession, use or copying. Consistent with FAR 12.211 and 12.212,*/
/*  Commercial Computer Software, Computer Software Documentation, and*/
/*  Technical Data for Commercial Items are licensed to the U.S. Government*/
/*  under vendor's standard commercial license.*/

#if !defined(_MATH_H_) && !defined(_MATH_H)
#define _MATH_H_
#define _MATH


/*  D.5 Standard C library headers*/

/*  For compatiblity with the Standard C library, the C++ Standard library*/
/*  provides the 18 C headers, as show in Table 100:*/

/*  ... Table includes <math.h>*/

/*  Each C header, whose name has the form name.h behaves as if each name*/
/*  placed in the Standard library namespace by the corresponding cname*/
/*  header is also placed within the namespace scope of the namespace std*/


#if !defined(_CMATH_)
#include <cmath>
#endif

/*  and is followed by an explicit using-declaration (7.3.3)*/

#if defined(__PURE_CNAME) || defined(__CNAME_OVERLOADS)
using std::abs;
#endif

using std::acos;
using std::asin;
using std::atan;
using std::atan2;
using std::ceil;
using std::cos;
using std::cosh;
using std::exp;
using std::fabs;
using std::floor;
using std::fmod;
using std::frexp;
using std::ldexp;
using std::log;
using std::log10;
using std::modf;
using std::pow;
using std::sin;
using std::sinh;
using std::sqrt;
using std::tan;
using std::tanh;


/*   Supporting Math Functions*/

/*  We can't provide these in -std ms mode because if we do we will*/
/*  get an compiler message that std::acosf and the acosf brought in*/
/*  by <math.h> conflict.  We can provide them in __PURE_CNAME mode*/
/*  since we never bring in <math.h> in that case.*/

#if (!defined(__STD_MS) && !defined(__MS)) || defined(__PURE_CNAME)
    using std::acosf;
    using std::asinf;
    using std::atanf;
    using std::atan2f;
    using std::ceilf;
    using std::cosf;
    using std::coshf;
    using std::expf;
    using std::fabsf;
    using std::floorf;
    using std::fmodf;
    using std::frexpf;
    using std::ldexpf;
    using std::logf;
    using std::log10f;
    using std::modff;
    using std::powf;
    using std::sinf;
    using std::sinhf;
    using std::sqrtf;
    using std::tanf;
    using std::tanhf;

# if __X_FLOAT
    using std::acosl;
    using std::asinl;
    using std::atanl;
    using std::atan2l;
    using std::ceill;
    using std::cosl;
    using std::coshl;
    using std::expl;
    using std::fabsl;
    using std::floorl;
    using std::fmodl;
    using std::frexpl;
    using std::ldexpl;
    using std::logl;
    using std::log10l;
    using std::modfl;
    using std::powl;
    using std::sinl;
    using std::sinhl;
    using std::sqrtl;
    using std::tanl;
    using std::tanhl;
# endif
#endif


/*  If __PURE_CNAME isn't set bring in math.h so we get whatever is*/
/*  declared/defined therein.*/

/*  If _FASTMATH is set, turn it off before bring in <math.h> since some*/
/*  system supplied <math.h>'s do not do so correctly.  We will implement*/
/*  is ourselves.  <math_cname_imple> does it for the standard required*/
/*  functions (cos, sin, etc).  We do it for the nonstandard functions*/
/*  that we get from math.h*/

#if !defined(__PURE_CNAME)
#   ifdef _FASTMATH
#	define __FASTMATH_SET _FASTMATH
#	undef _FASTMATH
#   endif

extern "C" {
#   undef _MATH_H_
#   undef _MATH_H

#   include_next <math.h>

#   if !defined(_MATH_H_)
#	define _MATH_H_
#   endif
#   if !defined(_MATH_H)
#	define _MATH_H
#   endif
}

#   ifdef __FASTMATH_SET
#	define _FASTMATH __FASTMATH_SET
#	undef __FASTMATH_SET
#	pragma __message __save
#	pragma __message __disable (errnointrinsic, undeclared)
#	pragma __intrinsic (cosd, cosdf, sind, sindf)
#	pragma __message __restore
#   endif
#endif

#endif
