/* Modifications Copyright 1997 Digital Equipment Corporation */
/*
 * @(#)typedefs_md.h	1.33 98/07/01
 *
 * Copyright 1995-1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 * 
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

/*
 * Solaris-dependent types for Green threads
 */

#ifndef _SOLARIS_TYPES_MD_H_
#define _SOLARIS_TYPES_MD_H_

#include <sys/types.h>
#include <sys/stat.h>
#include "bool.h"

/* don't redefine typedef's on Solaris 2.6 or Later */

#if !defined(_ILP32) && !defined(_LP64)

#ifndef	_UINT64_T
#define	_UINT64_T
typedef unsigned long uint64_t; /*** OSF/cds - was unsigned long long ***/
#define _UINT32_T
typedef unsigned int uint32_t; /*** OSF/cds - was unsigned long ***/
#endif

#ifndef	_INT64_T
#define	_INT64_T
typedef long int64_t; /*** OSF/cds - was long long ***/
#define _INT32_T
typedef int int32_t; /*** OSF/cds - was long ***/
#endif

#endif	/* !defined(_ILP32) && !defined(_LP64) */

/*** OSF/cds - short integer definitions added by OSF (16-bit and 8-bit) ***/
#ifndef _INT16_T
typedef short int16_t;
#define _INT16_T
#endif /* _INT16_T */

#ifndef _UINT16_T
typedef unsigned short uint16_t;
#define _UINT16_T
#endif /* _UINT16_T */

#ifndef _INT8_T
typedef signed char int8_t;     /* Plain char may be unsigned. */
#define _INT8_T
#endif /* _INT8_T */

#ifndef _UINT8_T
typedef unsigned char uint8_t;
#define _UINT8_T
#endif /* _UINT8_T */

/* *** OSF/cds - added by OSF
 *  An unsigned integer of the same size as void* to perform mask
 *  and unmaks on addresses.  An invalid pointer recognizable at
 *  a glance.
 */
#ifndef _UINT_PTR_T
typedef unsigned long uint_ptr_t;
#define _UINT_PTR_T
#endif /* _UINT_PTR_T */

#define INVALID_POINTER 0x5555555555555555

/*
 *  Safe conversion of pointers to an unsigned int of the same size,
 *  and vice versa.  Uses uint_ptr_t.
 */
#define p2u(p) ((uint_ptr_t)(p))
#define u2p(u) ((void*)(u))

/* use these macros when the compiler supports the long long type */

#define ll_high(a)    ((uint32_t)(((uint64_t)(a))>>32))  /*** OSF/cds - was (( long)((a)>>32)) ***/
#define ll_low(a)     ((uint32_t)(a))  /*** OSF/cds - was ((long)(a)) ***/
#define int2ll(a)	((int64_t)(a))
#define ll2int(a)	((int)(a))
#define ll_add(a, b)	((a) + (b))
#define ll_and(a, b)	((a) & (b))
#define ll_div(a, b)	((a) / (b))
#define ll_mul(a, b)	((a) * (b))
#define ll_neg(a)	(-(a))
#define ll_not(a)     (~(uint64_t)(a))    /*** OSF/cds - uint64_t cast added by OSF ***/
#define ll_or(a, b)   ((uint64_t)(a) | (b))    /*** OSF/cds - uint64_t cast added by OSF ***/
#define ll_shl(a, n)  ((uint64_t)(a) << (n))    /*** OSF/cds - uint64_t cast added by OSF ***/
#define ll_shr(a, n)  ((int64_t)(a) >> (n))    /*** cds - int64_t cast added ***/
#define ll_sub(a, b)  ((uint64_t)(a) - (b))    /*** OSF/cds - uint64_t cast added by OSF ***/
#define ll_ushr(a, n) ((unsigned long)(a) >>(n)) /*** cds - was long long ***/
#define ll_xor(a, b)  ((a) ^ (b))
#define uint2ll(a)    ((uint64_t)(a)) /*** OSF/cds - was ((uint64_t)(unsigned long)(a)) ***/
#define ll_rem(a,b)	((a) % (b))

#define INT_OP(x,op,y)  ((x) op (y))
#define NAN_CHECK(l,r,x) x
#define IS_NAN(x) isnan(x) /*** cds - was isnand (Sun syntax for double version? ***/


/* On Intel these conversions have to be method calls and not typecasts.
   See the win32 typedefs_md.h file */
#if defined(__i386__) || defined (__i386__)

extern int32_t float2l(float f);
extern int32_t double2l(double d);
extern int64_t float2ll(float f);
extern int64_t double2ll(double d);

#else /* not i386 */

#define float2l(f)	(f)
#define double2l(f)	(f)
#define float2ll(f)	((int64_t) (f))
#define double2ll(f)	((int64_t) (f))

#endif /* i386 */


#define ll2float(a)	((float) (a))
#define ll2double(a)	((double) (a))
#define double2ll(f)	((int64_t) (f))

/* comparison operators */
#define ll_ltz(ll)	((ll)<0)
#define ll_gez(ll)	((ll)>=0)
#define ll_eqz(a)	((a) == 0)
#define ll_eq(a, b)	((a) == (b))
#define ll_ne(a,b)	((a) != (b))
#define ll_ge(a,b)	((a) >= (b))
#define ll_le(a,b)	((a) <= (b))
#define ll_lt(a,b)	((a) < (b))
#define ll_gt(a,b)	((a) > (b))

#define ll_zero_const	((int64_t) 0)
#define ll_one_const	((int64_t) 1)

extern void ll2str(int64_t a, char *s, char *limit);

#ifdef ppc
#define HAVE_ALIGNED_DOUBLES
#define HAVE_ALIGNED_LONGLONGS
#endif

#endif /* !_SOLARIS_TYPES_MD_H_ */
