/*
 * *****************************************************************
 * *                                                               *
 * *    Copyright Compaq Computer Corporation, 2000                *
 * *                                                               *
 * *   The software contained on this media  is  proprietary  to   *
 * *   and  embodies  the  confidential  technology  of  Compaq    *
 * *   Computer Corporation.  Possession, use,  duplication  or    *
 * *   dissemination of the software and media is authorized only  *
 * *   pursuant to a valid written license from Compaq Computer    *
 * *   Corporation.                                                *
 * *                                                               *
 * *   RESTRICTED RIGHTS LEGEND   Use, duplication, or disclosure  *
 * *   by the U.S. Government is subject to restrictions  as  set  *
 * *   forth in Subparagraph (c)(1)(ii)  of  DFARS  252.227-7013,  *
 * *   or  in  FAR 52.227-19, as applicable.                       *
 * *                                                               *
 * *****************************************************************
 */
/*
 * HISTORY
 */
/*
 * @(#)$RCSfile: idset.h,v $ $Revision: 1.1.7.7 $ (DEC) $Date: 1999/11/30 16:07:25 $
 */

#ifndef _KERN_IDSET_H_
#define _KERN_IDSET_H_

typedef void *idset_t;

#ifndef NULL
		#undef NULL
#define NULL	0L
#endif

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

#ifndef BITS_PER_LONG
#define BITS_PER_LONG		(sizeof(long) * NBBY)
#endif

#ifndef SET_CURSOR_INIT
#define SET_CURSOR_INIT		0

#define SET_CURSOR_FIRST	1
#define SET_CURSOR_WRAP		2
#define SET_CURSOR_CONSUME	4
#endif

typedef unsigned int id_cursor_t, *id_cursor_p;

/*
 * The number of longs in a idset_t of __n__ elements
 */
#define IDSET_LSIZE(__n__)						\
	(((__n__) + ((BITS_PER_LONG) - 1)) / (BITS_PER_LONG))

/*
 * The maximum number of bits in a idset_t
 */
#define IDSET_NBITS(__n__)	IDSET_LSIZE(__n__) * BITS_PER_LONG

/*
 * Index into the embedded idset_t bits[] array for a given bit __i__
 */
#define IDSET_INDEX(__i__)	((__i__) / BITS_PER_LONG)

int idsetcreate(idset_t *idset_ptr, int max_members);
int idsetdestroy(idset_t *idset);
int idsetisvalid(idset_t idset);

int idcountset(idset_t idset);
int idemptyset(idset_t idset);
int idfillset(idset_t idset);
int idaddset(idset_t idset, int which);
int iddelset(idset_t idset, int which);

int idismember(idset_t idset, int which);
int idisemptyset(idset_t idset);

int idorset(idset_t set1, idset_t set2, idset_t dest_set);
int idxorset(idset_t set1, idset_t set2, idset_t dest_set);
int idandset(idset_t set1, idset_t set2, idset_t dest_set);
int iddiffset(idset_t set1, idset_t set2, idset_t dest_set);
int idcopyset(idset_t src_set, idset_t dest_set);

int id_foreach(idset_t idset, unsigned int flags, id_cursor_t *cursor);

#ifdef _KERNEL

/*
 * Magic numbers
 */
#define IDSET_HEADER_LSIZE       1
#define IDSET_HEADER_NBYTES      (IDSET_HEADER_LSIZE * sizeof(long))

#define __idsetcreate	idsetcreate
#define __idsetdestroy	idsetdestroy
#define __idsetisvalid	idsetisvalid
#define __idcountset	idcountset
#define __idemptyset	idemptyset
#define __idfillset	idfillset
#define __idaddset	idaddset
#define __iddelset	iddelset
#define __idismember	idismember
#define __idisemptyset	idisemptyset
#define __idorset	idorset
#define __idxorset	idxorset
#define __idandset	idandset
#define __iddiffset	iddiffset
#define __idcopyset	idcopyset
#define __id_foreach	id_foreach
#define __mask_to_idset	mask_to_idset
#define __idset_to_mask	idset_to_mask

#endif /* _KERNEL */

#endif /* _KERN_IDSET_H_ */
