/*
 * *****************************************************************
 * *                                                               *
 * *    Copyright Compaq Computer Corporation, 2000                *
 * *                                                               *
 * *   The software contained on this media  is  proprietary  to   *
 * *   and  embodies  the  confidential  technology  of  Compaq    *
 * *   Computer Corporation.  Possession, use,  duplication  or    *
 * *   dissemination of the software and media is authorized only  *
 * *   pursuant to a valid written license from Compaq Computer    *
 * *   Corporation.                                                *
 * *                                                               *
 * *   RESTRICTED RIGHTS LEGEND   Use, duplication, or disclosure  *
 * *   by the U.S. Government is subject to restrictions  as  set  *
 * *   forth in Subparagraph (c)(1)(ii)  of  DFARS  252.227-7013,  *
 * *   or  in  FAR 52.227-19, as applicable.                       *
 * *                                                               *
 * *****************************************************************
 */
/*
 * HISTORY
 */
/*	
 *	@(#)$RCSfile: netisr.h,v $ $Revision: 4.2.45.7 $ (DEC) $Date: 2000/03/17 15:13:03 $
 */
/*
 * (c) Copyright 1990, OPEN SOFTWARE FOUNDATION, INC.
 * ALL RIGHTS RESERVED
 */
/*
 * OSF/1 Release 1.0
 */
/*
 * Copyright (C) 1988,1989 Encore Computer Corporation.  All Rights Reserved
 *
 * Property of Encore Computer Corporation.
 * This software is made available solely pursuant to the terms of
 * a software license agreement which governs its use. Unauthorized
 * duplication, distribution or sale are strictly prohibited.
 *
 */
/*
 * Copyright (c) 1980, 1986, 1989 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted provided
 * that: (1) source distributions retain this entire copyright notice and
 * comment, and (2) distributions including binaries display the following
 * acknowledgement:  ``This product includes software developed by the
 * University of California, Berkeley and its contributors'' in the
 * documentation or other materials provided with the distribution and in
 * all advertising materials mentioning features or use of this software.
 * Neither the name of the University nor the names of its contributors may
 * be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *	Base:	netisr.h	7.5 (Berkeley) 4/22/89
 *	Merged:	netisr.h	7.6 (Berkeley) 6/28/90
 */

#ifndef	_NET_NETISR_H
#define	_NET_NETISR_H

#ifndef REPLICATED
#define REPLICATED
#endif

/*
 * The networking code is in separate kernel threads or in software
 * interrupts. When running in threads, all events are delivered in
 * thread context, at splnet() or spl0() depending on lock configuration.
 * When running in software interrupts, events are delivered at splnet()
 * or splsoftclock().
 *
 * Clients of the isr framework use a "netisr" structure to
 * enqueue events and maintain active status. Some default
 * structures are defined globally (especially those used in
 * non-local contexts e.g. packet receive), others may be
 * registered with netisr_add(). See net/netisr.c.
 */

struct netisr {
	/* 64-byte cache alignment, read-mostly data */
	struct	netisr *next, *prev;	/* Link in {in}active isr list */
	void	(*isr)(void);		/* Isr to process input */
	struct	ifqueue *ifq;		/* Queue to receive packets, or NULL */
	struct	domain *dom;		/* Domain isr belongs to, or NULL */
	int	id;			/* Integer id */
	int pad[5];
	/* 64-byte cache alignment, statistics */
	int	active;			/* Softnet active count */
	int	pending;		/* Interrupt pending on queue */
	u_short	flags;			/* Flags */
	u_short use;  			/* use count for this netisr */
#define ISRF_INCHDR	1		/*   Include datalink */
	short	unit;			/* unit number for NETISR_NETDEV */
	short	pad_a;
	u_int	events;			/* Event count */
	u_int	wakeups;		/* Wakeup count */
	u_int	extras;			/* Wakeups with nothing to do */
	u_int	noserver;		/* No servers ready - NETISR_THREAD */
	u_int	block;			/* cnt threads blocked-NETISR_THREAD */
	u_int	maxactive;		/* max # concurrent threads active */
	int pad_b[6];
};

/*
 * Isr's may be used to deliver data to protocols, or simply events.
 */

#define NETISR_WILD	0	/* Copy of all packets */
#define NETISR_OTHER	1	/* Packets otherwise tossed (->dlpi)*/
#define	NETISR_ARP	2	/* ARP packets */
#define	NETISR_IP	3	/* IP packets */
#define	NETISR_RTMSG	4	/* Internal routing messages */
#define	NETISR_ISO	5	/* ISO packets */

#define	NETISR_MB	6	/* Mbuf/mblk event */
#define NETISR_STREAMS	7	/* Streams scheduler */
#define NETISR_ATMRTUPT	8	/* ATM IP route update */
#define NETISR_STRWELD	9	/* Streams "weld" */
#define NETISR_PFFAST	10	/* Sockets fasttimeout */
#define NETISR_PFSLOW	11	/* Sockets slowtimeout */
#define NETISR_IFSLOW	12	/* Sockets interface watchdog */
#define NETISR_ARPTMO	13	/* Sockets ARP timer */

#define NETISR_EVL	14	/* X.25 event logging */
#define NETISR_DLO	15	/* used for node 0 in DECnet */
#define	NETISR_DN	16	/* DECnet packets */
#define	NETISR_DLI	17	/* DLI packets */
#define	NETISR_LAT	18	/* LAT packets */
#define NETISR_WDD	19	/* WAN packets */
#define NETISR_NFSTMO	20	/* NFS timer */

#define NETISR_SRTMO	21	/* Token Ring S.R. timer */
#define NETISR_X25	22	/* X.25 packets */
#define NETISR_HDLC	23	/* HDLC packets */
#define NETISR_CTF	24	/* CTF messages */
#define NETISR_AAL	25	/* "Native" ATM API packets */
#define NETISR_IPV6	26	/* IPV6 packets */
#define NETISR_NETRAIN	27	/* Netrain failure detection code */
#define NETISR_NETDEV	28	/* network driver interrupt threads */

#define NETISR_MAX	40	/* maximum acticipated number of netisr id's */

/* The number of preallocated isr's is arbitrary, but
 * should be large enough for the above predefines. */
#define NNETISR		NETISR_MAX

#ifdef	_KERNEL

#if	NETISR_THREAD

extern REPLICATED int netisrwakeupthreshold;

#else	/* NETISR_THREAD */

#if defined(__vax__) || defined(__tahoe__)
#include "machine/mtpr.h"
#define setsoftnet()    mtpr(SIRR, 12)
#else
extern void setsoftnet(void);
#endif

#endif	/* NETISR_THREAD */

#define schednetisr(num) do {	\
	int _s = splimp();	\
	schednetisr_nospl(num);	\
	splx(_s);		\
} while (0)

extern struct netisr **netisr_active_rad;

#if	NETSYNC_LOCK
/* Next 3 macros use the per-RAD netisr_slock, and protect that RAD only */
#define NETISR_LOCKINIT_RAD(rad) simple_lock_setup(&rad->netisr_slock, netisr_li)
#define NETISR_LOCK_RAD(rad)	simple_lock(&rad->netisr_slock)
#define NETISR_UNLOCK_RAD(rad)	simple_unlock(&rad->netisr_slock)
#else
#define NETISR_LOCKINIT_RAD()
#define NETISR_LOCK_RAD()
#define NETISR_UNLOCK_RAD()
#endif
#endif
#endif
