/*
 * *****************************************************************
 * *                                                               *
 * *    Copyright Compaq Computer Corporation, 2000                *
 * *                                                               *
 * *   The software contained on this media  is  proprietary  to   *
 * *   and  embodies  the  confidential  technology  of  Compaq    *
 * *   Computer Corporation.  Possession, use,  duplication  or    *
 * *   dissemination of the software and media is authorized only  *
 * *   pursuant to a valid written license from Compaq Computer    *
 * *   Corporation.                                                *
 * *                                                               *
 * *   RESTRICTED RIGHTS LEGEND   Use, duplication, or disclosure  *
 * *   by the U.S. Government is subject to restrictions  as  set  *
 * *   forth in Subparagraph (c)(1)(ii)  of  DFARS  252.227-7013,  *
 * *   or  in  FAR 52.227-19, as applicable.                       *
 * *                                                               *
 * *****************************************************************
 */
/*
 * HISTORY
 */
/***** Until this works its commented out dupdef in Date variable
***#ifndef lint
***static char *rcsid = "@(#)$RCSfile: sia.h,v $ $Revision: 1.1.50.2 $ (DEC) $Date: 2000/05/03 19:29:42 $";
***#endif
***************/
/***********************************************************************
*	sia.h - SIA constants, structures, and macro definitions 
************************************************************************/
#ifndef _SIA_H_
#define _SIA_H_
#ifdef __cplusplus
extern "C" {
#endif

/***** sia common sys includes **/
#include <sys/param.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <sys/file.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <sys/sysinfo.h>
#include <sys/utsname.h>
#include <sys/sem.h>
#include <sys/uio.h>
#include <sys/audit.h>

/***** sia common usr includes **/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sgtty.h>
#include <utmp.h>
#include <signal.h>
#include <pwd.h>
#include <grp.h>
#include <lastlog.h>
#include <errno.h>
#include <ttyent.h>
#include <syslog.h>
#include <limits.h>
#include <strings.h>
#include <loader.h>
#include <nl_types.h>

#ifdef _THREAD_SAFE
#ifndef __BRAVO_DISABLED__
#include <cma.h>
#endif
#include <pthread.h>
#endif

/**** sia logical definitions ****/

#ifndef	NULL
		#undef NULL
#define NULL 0L
#endif
#ifndef	YES
#define YES 1
#endif
#ifndef	NO
#define NO 0
#endif
#ifndef	TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif

/**** sia library and filenames ****/

#define SIALOGFILE      "/var/adm/sialog"          /* SIA logging file       */
#define SIAIGOODFILE    "/etc/sia/siainitgood"     /* siainit good file flag */
#define MATRIX_CONF     "/etc/sia/matrix.conf"     /* sia matrix config file */
#define SIALIBCSO       "libc.so"                  /* local security in libc */

/***** sia limits ***********/

#define SIABUFSIZ       1024    /* buffer size for loginname, password, and  */
                                /* one line of the SIA matrix config file    */
#define SIAPKGNAMAX     64      /* Libsia packagename maximum size           */
#define SIALIBNAMAX     64      /* Libsia packagename maximum size           */
/* This next is obsolete -- the sia_caps array is NULL-terminated.  See also
 * __sia_cap_max (in siad.h).
 */
#define SIADCAPMAX      32      /* max sec mech sia capabilities             */
/* This next is obsolete -- If you really need to know, see __sia_sw_max     */
#define SIASWMAX        4       /* max switches per capability               */
#define SIAMUTMAX       16      /* number mutexes utilized by SIA            */
#define SIAMXACCTNAME   64      /* maximum size of an account name           */
#define SIAMXPASSWORD   80      /* maximum size of a password                */
#define SIANAMEMIN      16      /* minimum space allocated for entity->name  */


typedef struct sia_ai_packet {
        int  token;           /* tkn number */
                /*equate the token number with the msg cat and the
                * look up by type, see SIA_T_IDX.
                */
        int  toktype;		/* token data type */
        void *tokval;		/* token data */
	char tokstat;
        char **responders;      /* list of strings of who answered. */
        char *litstring;        /* see prot.h for definitions */
        char *msglabel;
        char *msghelp;
        void *reserved1;
        void *reserved2;
        void *reserved3;
        void *reserved4;
        void *reserved5;
}SIA_AI_PKT;

/* sia_attr_query() block allocates SIA_E_AI_LIST[]. reserved[1-5] provide
*  future expansion/backwards compatility.
*/ 

/* responders field components/answers */
#define SIA_T_BSD_STR "bsd"
#define SIA_T_EHNANCED_STR "enhanced"
#define SIA_T_DB_DEFAULT 'd'
#define SIA_T_DB_PRIMARY 'p'
#define SIA_T_DB_NONE 'n'
#define SIA_T_DB_UNKNOWN '?'

#define SIA_T_DBA_PRIMARY 0x1
#define SIA_T_DBA_DEFAULT 0x2

#define IS_PDBA(dba)((dba) & SIA_T_DBA_PRIMARY)
#define IS_DDBA(dba)((dba) & SIA_T_DBA_DEFAULT)

/* see SIA_T_IDX */
#define SIA_IDX_BSD 1
#define SIA_IDX_ENHANCED 2

/* token type_values */
#define SIA_TT_INT              0x1
#define SIA_TT_PPSTR            0x2
#define SIA_TT_CHAR             0x3
#define SIA_TT_TIME             0x4
#define SIA_TT_UID              0x5
#define SIA_TT_GID              0x6
#define SIA_TT_UINT             0x7
#define SIA_TT_UCHAR            0x8
#define SIA_TT_ULONG            0x9
#define SIA_TT_LONG             0xA
#define SIA_TT_MAND_IR		0xB  /* no op, TBI */

/*#define SIA_TT_MASKT
* #define SIA_TT_PRIV  
*/

/* what field to fill in or remove for [qgs]attr response parameter*/
#define SIA_AI_ALL              0xFFFFFFFFF
#define SIA_AI_TOKEN            0x1
#define SIA_AI_RESPONDER        0x2  /* responders, cleans out tokval too */
#define SIA_AI_LITSTR           0x4  /* literal  string */
#define SIA_AI_MSGLABEL         0x8
#define SIA_AI_MSGHELP          0x10  /* help message */
#define SIA_AI_TOKENTYPE        0x100 /* token data type */
#define SIA_AI_VALUE            0x200 /* attribute value */
#define SIA_AI_BLKZERO          0x400 /* zero the ai_list*/
#define SIA_AI_BLKFREE          0x800 /* free the ai_list, must zero if free */
#define SIA_AI_FLIST            SIA_AI_MSGLABEL|SIA_AI_MSGHELP|SIA_AI_RESPONDER
#define SIA_AI_KEEPZBLK         SIA_AI_MSGLABEL|SIA_AI_MSGHELP|SIA_AI_RESPONDER|SIA_AI_BLKZERO


/* Token status */
#define SIA_TS_UNANSWERED '\000'
#define SIA_TS_ANSWERED 'a'
#define SIA_TS_AMBIGUOUS 'd'
#define SIA_TS_NOTSET 'n'
#define SIA_TS_UNKNOWN '?'

/* number of possible token types */
#define SIA_TT_LIMIT	1024

/* number of tokens that can be used at one time -1 */
#define SIA_NT_LIMIT	1024

/* TU - token user db */
typedef enum sia_t_idx {	/* /etc/passwd first */
   SIA_TU_PWNAME=1,		/* char *pw_pwname, fd_name */
   SIA_TU_PWD,			/* char *pw_pwd, fd_encrypt */
   SIA_TU_UID,			/* uid_t pw_uid, fd_uid */
   SIA_TU_GID,			/* gid_t pw_gid */
   SIA_TU_COMMENT,		/* char * for future SysV compatibility */
   SIA_TU_QUOTA,		/* int for future SysV compatibility */
   SIA_TU_GECOS,		/* char *pw_gecos */
   SIA_TU_DIR,			/* char *pw_dir */
   SIA_TU_SHELL,		/* char *pw_shell */
   SIA_TU_OWNER,		/* char *, start enhanced 'd' indicates defaults also */
   SIA_TU_PRIORITY,		/* int fd_nice */
   SIA_TU_CMDPRIV,		/* char *, NOT mask_t * */
   SIA_TU_SYSPRIV,		/* char *, NOT priv_t * */
   SIA_TU_BASEPRIV,		/* char *, NOT priv_t * */
   SIA_TU_AUDITDISP,		/* char * */
   SIA_TU_AUDITCNTL,		/* uint_t */
   SIA_TU_MINCHG,		/* time_t */
   SIA_TU_MINLEN,		/* int */
   SIA_TU_MAXLEN,		/* int */
   SIA_TU_EXP,			/* time_t fd_expire */
   SIA_TU_LIFE,			/* time_t fd_lifetime */
   SIA_TU_SUCCHG,		/* time_t fd_schange */
   SIA_TU_UNSUCCHG,		/* time_t fd_uchange */
   SIA_TU_PICKPWD,		/* char fd_pick_pwd */
   SIA_TU_GENPWD,		/* char fd_gen_pwd */
   SIA_TU_RESTRICT,		/* char */ 
   SIA_TU_POLICY,		/* char */
   SIA_TU_NULLPW,		/* char */
   SIA_TU_PWCHANGER,		/* uid_t */
   SIA_TU_GENCHARS,		/* char fd_gen_chars */
   SIA_TU_GENLETTERS,		/* char fd_gen_letters */
   SIA_TU_PWDEPTH,		/* int */
   SIA_TU_PWDICT,		/* char * */ 
   SIA_TU_OLDCRYPT,		/* char */
   SIA_TU_NEWCRYPT,		/* char */
   SIA_TU_CLEARANCE,		/* mand_ir_t * */
   SIA_TU_SUCLOG,		/* time_t fd_slogin */
   SIA_TU_UNSUCLOG,		/* time_t fd_ulogin */
   SIA_TU_SUCTTY,		/* char * */
   SIA_TU_NUMUNSUCLOG,		/* int fd_nlogins */
   SIA_TU_UNSUCTTY,		/* char * */
   SIA_TU_TOD,			/* char * */
   SIA_TU_MAXTRIES,		/* int fd_max_tries */
   SIA_TU_UNLOCK,		/* time_t fd_unlockint */
   SIA_TU_RETIRED,		/* char */
   SIA_TU_LOCK,			/* char */
   SIA_TU_EXPDATE,		/* time_t */
   SIA_TU_ISTEMPLATE,		/* char */
   SIA_TU_TEMPLATE,		/* char * */
   SIA_TU_VAC_START,		/* time_t */
   SIA_TU_VAC_END,		/* time_t */
   SIA_TU_RLIMIT_CPU,		/* ulong_t fd_rlim_cpu */
   SIA_TU_RLIMIT_FSIZE,		/* ulong_t fd_rlim_fsize */
   SIA_TU_RLIMIT_DATA,		/* ulong_t fd_rlim_data */
   SIA_TU_RLIMIT_STACK,		/* ulong_t fd_rlim_stack */
   SIA_TU_RLIMIT_CORE,		/* ulong_t fd_rlim_core */
   SIA_TU_RLIMIT_RSS,		/* ulong_t fd_rlim_rss */
   SIA_TU_RLIMIT_NOFILE,	/* ulong_t fd_rlim_nofile */
   SIA_TU_RLIMIT_VMEM,		/* ulong_t fd_rlim_vmem */
   SIA_TU_MAX_LOGIN_INT,	/* time_t */
   SIA_TU_GRACE_LIMIT,		/* time_t */
   SIA_TU_PSW_CHG_REQD,		/* char */
   SIA_TU_GEN_SITE,		/* char */
   SIA_TU_MINCHOSEN,		/* uchar_t */
   SIA_TU_MAXCHOSEN,		/* uchar_t */
   SIA_TU_FLOGINS,		/* int */
   SIA_TU_PWDSTAT		/* fake token */
} SIA_T_IDX;


/**** Separate expansion sia entity structure ****/

typedef struct siaentitytoo {
			unsigned char *orig_amask;  /* audit mask pre-sia_ses_init */
			int orig_acntl;		    /* audit control flags pre-sia_ses_init */
			SIA_AI_PKT **ai_list;
			char *pw_gname;
			/* May be extended -- don't rely on this length. */
			} SIAENTITYTOO;
/* macros which can be changed to access private info */
#define SIA_E_AI_LIST entity->siaetoo->ai_list
#define SIA_E_PW_GNAME entity->siaetoo->pw_gname

/***** sia entity structure *******/ 

typedef struct siaentity {
                        char *name;             /* collected name            */
			char *password;         /* entered or collected password */
                        char *acctname;         /* account name              */
                        char **argv;            /* calling command argument  */
                                                /* strings                   */
                        int  argc;              /* number of arguments       */
			uid_t ouid;		/* original UID		     */
                        char *hostname;         /* requesting host NULL=>local */
                        char *tty;              /* pathname of local tty     */
                        int colinput;           /* 1 => yes 0 => no input    */
                        int error;              /* error message value       */
                        int authcount;          /* Number of consecutive     */
                                                /* Failed authen attempts    */
                        int authtype;           /* Type of last authent      */
                        struct passwd *pwd;     /* pointer to pwent struct   */
			char *gssapi;
                        SIAENTITYTOO *siaetoo;	/* pointer to SIAENTITYTOO,
						 * formerly the previously
						 * unused and reserved field
						 * char *sia_pp */
                        int *mech[SIASWMAX];    /* pointers to mechanism     */
                                                /* specific data             */
						/* (array may be longer --   */
						/* see __sia_sw_max)	     */
                        } SIAENTITY;


/***** sia collect routine interface definition *****/

typedef struct prompt_t {
        unsigned char *prompt;
        unsigned char *result;
        int max_result_length;
        int min_result_length;
        int control_flags;
} prompt_t;

typedef int sia_collect_func_t __((int /* timeout */, int /* rendition */,
					unsigned char * /* title */,
					int /* num_prompts */,
					prompt_t * /* prompts */));

/* top level returns from sia routines */

#define SIASUCCESS      1  		/* success return */
#define SIAFAIL         0 		/* failure return */
#define SIASTOP		2		/* stop processing */


/**************************************/
/***** sia top level interface    *****/
/***** special purpose routines   *****/
/**************************************/

extern int sia_init __((void));		/* Called to ensure that matrix.conf */
					/* has been read.		     */
extern int __sia_set_matconf __((char const *));
					/* Called to allow the use of an     */
					/* alternate matrix.conf file.	     */

extern int sia_authorize __((SIAENTITY *)); /* only called internally */
					     /* for sia authorization */

extern int sia_chk_invoker __((void));	     /* called to check if process is */
					     /* sufficiently privileged	      */

extern int sia_get_groups __((const char *, gid_t, gid_t *, int *)); /* initgroups processing, short of setgroups() */

sia_collect_func_t sia_collect_trm;	/* general collection routine for*/
					/* terminals */

extern int sia_free_entity __((SIAENTITY *, int));

/**************************************/
/***** sia change secure info calls ***/
/**************************************/

extern int sia_chg_finger __((sia_collect_func_t *, char *, int, char *[]));
extern int sia_chg_password __((sia_collect_func_t *, char *, int, char *[]));
extern int sia_chg_shell __((sia_collect_func_t *, char *, int, char *[]));
extern int sia_update_pass __((sia_collect_func_t *,
			       SIAENTITY *, const char *));

/*************************************/
/**** session processing calls	******/
/*************************************/


extern int sia_ses_init __((SIAENTITY **, int, char *[], char *, char *, char *, int, char *));
extern int sia_ses_authent __((sia_collect_func_t *, char *, SIAENTITY *));
extern int sia_ses_reauthent __((sia_collect_func_t *,  SIAENTITY *));
extern int sia_ses_suauthent __((sia_collect_func_t *,  SIAENTITY *));
extern int sia_ses_estab __((sia_collect_func_t *, SIAENTITY *));
extern int sia_ses_launch __((sia_collect_func_t *, SIAENTITY *));
extern int sia_ses_release __((SIAENTITY **));
extern int sia_attr_getnam __((SIAENTITY *, char *, SIA_AI_PKT **, uint_t, uint_t));
extern int sia_attr_query __((SIAENTITY *, SIA_AI_PKT **, size_t, int *));

extern int sia_attr_setent __((SIAENTITY *));
extern int sia_attr_getent __((SIAENTITY *, SIA_AI_PKT **, uint_t, uint_t));
extern int sia_attr_putent __((SIAENTITY *, SIA_AI_PKT **));
extern int sia_attr_endent __((SIAENTITY *));
extern int sia_attr_to_string __((SIA_AI_PKT **, char **, size_t *, const char));


/*********************************************/
/**** streamlined session processing calls ***/
/*********************************************/

#define	SIA_BEU_CLEARENV	0x001
#define	SIA_BEU_SETLUID		0x002
#define	SIA_BEU_DOCHDIR		0x004
#define	SIA_BEU_OKROOTDIR	0x008	/* ignored without *_DOCHDIR */
#define	SIA_BEU_REALLOGIN	0x010	/* do ses_estab & such, not su-style */
/* REALLOGIN implies DOCHDIR and SETLUID and sort-of implies OKROOTDIR */

extern int sia_become_user __((sia_collect_func_t *, int, char *[], char *, char *, char *, int, char *, char **, int));
extern int sia_validate_user __((sia_collect_func_t *, int, char *[], char *, char *, char *, int, char *, char *));

#ifdef __cplusplus
}
#endif
#endif /* _SIA_H_ */
